# -*- Mode: python -*-
#
# cmd_addtopl.py - addtopl command
#
# Copyright (C) 2003, Xiph.org Foundation
# Copyright (C) 2003 Brett Smith <bretts@canonical.org>
#
# This file is part of positron.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of a BSD-style license (see the COPYING file in the
# distribution).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the license for more details.

"""positron addtopl:\tAdds files to a Neuros playlist.

   positron addtopl <playlist> <file1> <file2> ...

      Adds the given files in order to the playlist with the given name.

If a playlist with the specified name does not exist, it will be created.
Files listed should be files on the Neuros.
"""

import os

def run(config, neuros, args):
    playlist_name = args.pop(0)

    audio_db = neuros.open_db('audio')
    pl_db = audio_db.children[0]

    filelist = []
    for filename in args:
        filename = os.path.abspath(filename)
        if not neuros.is_valid_hostpath(filename):
            print ("Warning: Ignoring %s because it is not on the Neuros."
                   % filename)
        neuros_file = neuros.hostpath_to_neurospath(filename)
        record = audio_db.find(neuros_file, 8)
        if record is not None:
            mdb_ptr = audio_db.sai[record][0]
            filelist.append((neuros_file, mdb_ptr))
        else:
            print ("Warning: Ignoring %s; it is not in the database."
                   % filename)

    num_files = len(filelist)
    if num_files < 1:
        print "No valid filenames given to add!"
    else:
        playlist_sai = pl_db.find(playlist_name)
        if playlist_sai is None:
            print "Creating playlist: %s...\n" % playlist_name
            playlist_pai = pl_db._add_record((playlist_name,))[1]
        else:
            print "Adding to playlist: %s...\n" % playlist_name
            playlist_pai = pl_db.sai[playlist_sai][1]
        print "Adding %s tracks to the playlist..." % num_files
        for filename, mdb_ptr in filelist:
            print "  %s" % os.path.split(filename)[1]
            pl_db.pai.add_entry_to_module_at(playlist_pai, mdb_ptr)
        print "\nDone!"
