# -*- Mode: python -*-
#
# cmd_editpl.py - editpl command
#
# Copyright (C) 2003 Brett Smith <bretts@canonical.org>
#
# This file is part of positron.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of a BSD-style license (see the COPYING file in the
# distribution).
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the license for more details.

"""positron editpl:\tEdit a Neuros playlist.

   positron editpl <playlist>

      Invokes an editor to let you move and remove tracks in a Neuros
      playlist.

"""

import db

import os
import re
import tempfile

def get_playlist_tracks(playlist_name, audio_db):
    pl_db = audio_db.children[0]

    playlist_sai = pl_db.find(playlist_name)
    if playlist_sai is None:
        raise ValueError("Playlist not found: %s" % playlist_name)
    else:
        playlist_pai = pl_db.sai[playlist_sai][1]
        mdb_ptrs = [entry[0] for entry in
                    pl_db.pai.read_module_at(playlist_pai)[0]]
        return playlist_pai, mdb_ptrs
    

def test_mdb_record(mdb, pointer):
    try:
        mdb.read_record_at(pointer)
    except db.util.Error:
        return 0
    return 1


def run(config, neuros, args):
    playlist_name = args.pop(0)

    audio_db = neuros.open_db('audio')

    try:
        playlist_pai, mdb_ptrs = get_playlist_tracks(playlist_name, audio_db)
    except ValueError, error:
        print error
        return

    filename = tempfile.mktemp()
    try:
        trackfile = file(filename, 'w')
        trackfile.write("# Move lines in this file to sort the playlist.\n")
        trackfile.write("# Remove lines from this file to remove tracks.\n")
        trackfile.write("# Any other edits may produce bad results.\n\n")
        for ptr in mdb_ptrs:
            record = audio_db.mdb.read_record_at(ptr)[0]
            trackfile.write("%s (%s)\n" % (record['data'][0], ptr))
        trackfile.close()
    except IOError, error:
        print "Could not make tracklist file (%s)!" % error
        return

    editor = os.environ.get('VISUAL', None)
    if editor is None:
        editor = os.environ.get('EDITOR', None)
        if editor is None:
            editor = '/usr/bin/editor'
    result = os.system("%s %s" % (editor, filename))
    if result != 0:
        print "Tracklist editor did not return properly!"
        return

    mdb_re = re.compile(r'^.*\s\((\d+)\)$')
    mdb_ptrs = []
    try:
        trackfile = file(filename)
        while 1:
            line = trackfile.readline()
            if not line:
                break
            match = mdb_re.match(line)
            if match:
                mdb_ptrs.append(int(match.group(1), 10))
        trackfile.close()
    except IOError, error:
        print "Could not read tracklist file (%s)!" % error
        return

    filter(lambda ptr: test_mdb_record(audio_db.mdb, ptr), mdb_ptrs)
    if not mdb_ptrs:
        print "No valid tracklist records found!"
        return
    print "Writing new tracklist to playlist: %s..." % playlist_name
    playlist_db = audio_db.children[0]
    playlist_db.pai.clear_module_at(playlist_pai)
    for ptr in mdb_ptrs:
        playlist_db.pai.add_entry_to_module_at(playlist_pai, ptr)
    print "\nDone!"
    
